"use client";
import Image from "next/image";
import * as React from "react";
import { useState } from "react";

import { zodResolver } from "@hookform/resolvers/zod";
import { Loader2, Upload } from "lucide-react";
import { useLocale, useTranslations } from "next-intl";
import { useDropzone } from "react-dropzone";
import { useForm } from "react-hook-form";
import * as z from "zod";

import { Button } from "@/components/ui/button";
import { CalendarWithSearch } from "@/components/ui/calendar";
import {
	Form,
	FormControl,
	FormField,
	FormItem,
	FormMessage,
} from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { Label } from "@/components/ui/label";
import PhoneInput from "@/components/ui/phone-input";
import { PlaceholdersAndVanishInput } from "@/components/ui/placeholders-and-vanish-input";
import { Popover, PopoverContent, PopoverTrigger } from "@/components/ui/popover";
import {
	Select,
	SelectContent,
	SelectItem,
	SelectTrigger,
	SelectValue,
} from "@/components/ui/select";
import nationalities from "@/data/nationalities.json";
import { cn } from "@/lib/utils";
import { format } from "date-fns";
import { CalendarIcon } from "lucide-react";
import { toast } from "sonner";

import { submitRequestForEmploymentForm } from "./actions";

const genders = [
	{ en: "female", ar: "أنثى" },
	{ en: "male", ar: "ذكر" },

];


type EmploymentFormValues = {
	fullName: string;
	email: string;
	phone: string;
	expertCertificates: File | null;
	professionalLicenses: File | null;
	qualifications: File | null;
	yearsOfExperience: string;
	cv: File | null;
	address: string;
	startDate: string;
	expectedSalary: string;
	gender: "male" | "female";
	nationality: string;
	dateOfBirth: string;
};

export default function EmploymentForm() {
	const t = useTranslations("RequestForEmployment.EmploymentForm");
	const locale = useLocale();
	const [isLoading, setIsLoading] = React.useState(false);
	const [searchQuery, setSearchQuery] = React.useState("");

	const formSchema = z.object({
		fullName: z.string().min(1, t("validation.fullName.required")),
		email: z
			.string()
			.email(t("validation.email.invalid"))
			.min(1, t("validation.email.required")),
		phone: z.string().min(1, t("validation.phone.required")),
		expertCertificates: z.custom<File>((val) => val instanceof File, t("validation.expertCertificates.required")),
		professionalLicenses: z.custom<File>((val) => val instanceof File, t("validation.professionalLicenses.required")),
		qualifications: z.custom<File>((val) => val instanceof File, t("validation.qualifications.required")),
		yearsOfExperience: z
			.string()
			.min(1, t("validation.yearsOfExperience.required"))
			.refine((val) => !isNaN(Number(val)), {
				message: t("validation.yearsOfExperience.invalid")
			})
			.refine((val) => Number(val) >= 0, {
				message: t("validation.yearsOfExperience.invalid")
			})
			.transform((val) => Number(val)),
		cv: z.custom<File>((val) => val instanceof File, t("validation.cv.required")),
		address: z.string().min(1, t("validation.address.required")),
		startDate: z.string().min(1, t("validation.startDate.required")),
		expectedSalary: z
			.string()
			.min(1, t("validation.expectedSalary.required"))
			.refine((val) => !isNaN(Number(val)), {
				message: t("validation.expectedSalary.invalid")
			})
			.refine((val) => Number(val) >= 0, {
				message: t("validation.expectedSalary.invalid")
			})
			.transform((val) => Number(val)),
		gender: z.enum(["male", "female"]),
		nationality: z.string().min(1, t("validation.nationality.required")),
		dateOfBirth: z.string().min(1, t("validation.dateOfBirth.required")),
	});

	const [files, setFiles] = useState({
		cv: null,
		expertCertificates: null,
		professionalLicenses: null,
		qualifications: null,
	});

	const onDrop = (name: keyof typeof files) => (acceptedFiles: File[]) => {
		const file = acceptedFiles[0] || null;
		setFiles((prev) => ({ ...prev, [name]: file }));
		form.setValue(name, file);
	};

	const FileField: React.FC<{ name: keyof typeof files; label: string }> = ({
		name,
		label

	}) => {
		const { getRootProps, getInputProps } = useDropzone({
			onDrop: onDrop(name),
			accept: {
				"application/pdf": [".pdf"],
				"application/msword": [".doc", ".docx"],
				"image/*": [".jpg", ".jpeg", ".png", ".gif"],
			},
		});

		const isImage = (file: File | null) => {
			return file && file.type.startsWith("image/");
		};

		return (
			<FormField
				control={form.control}
				name={name}
				render={() => (
					<FormItem>
						<Label>
							<p className="text-base leading-6 text-black">
								{label} <span className="text-[#EB5757]">*</span>
							</p>
						</Label>
						<FormControl>
							<div>
								<div
									{...getRootProps()}
									className="cursor-pointer rounded-md border border-[#DEDEDE] p-4"
								>
									<input {...getInputProps()} />
									<div className="flex items-center gap-2">
										<Upload className="h-5 w-5 black" />
										<p className="text-[14px] text-neutral-400">
											{
												// eslint-disable-next-line @typescript-eslint/ban-ts-comment
												// @ts-expect-error
												files[name]?.name || t("Placeholder.UploadFile")
											}
										</p>
									</div>
								</div>
								{isImage(files[name]) && files[name] && (
									<div className="mt-2">
										<Image
											src={URL.createObjectURL(files[name] as File)}
											alt="Preview"
											width={128}
											height={128}
											className="h-32 w-32 rounded-md border object-cover"
										/>
									</div>
								)}
							</div>
						</FormControl>
						<FormMessage />
					</FormItem>
				)}
			/>
		);
	};


	const form = useForm<EmploymentFormValues>({
		resolver: zodResolver(formSchema),
		defaultValues: {
			fullName: "",
			email: "",
			phone: "",
			expertCertificates: null,
			professionalLicenses: null,
			qualifications: null,
			yearsOfExperience: "",
			cv: null,
			address: "",
			startDate: "",
			expectedSalary: "",
			gender: "male",
			nationality: "",
			dateOfBirth: "",
		},
	});

	const renderFormField = (
		name: keyof EmploymentFormValues,
		label: string,
		placeholder?: string,
		type: string = "text",
		options?: any,
		required: boolean = true,
		withLabel = true,
		icon?: string,
		filterable: boolean = false
	) => {

		return (
			<FormField
				control={form.control}
				name={name}
				render={({ field }) => (
					<FormItem>
						{withLabel && (
							<Label>
								<p className="#000 text-base leading-6">
									{label}
									{required && <span className="text-[#EB5757]"> *</span>}
								</p>
							</Label>
						)}
						<FormControl>
							{type === "select" ? (
								<Select
									onValueChange={field.onChange}
									defaultValue={field.value as string}
									onOpenChange={() => setSearchQuery("")}
									dir={locale === "ar" ? "rtl" : "ltr"}
								>
									<SelectTrigger className={cn(
										"h-[56px] w-full rounded-none border-[#DEDEDE]",
										filterable && "text-neutral-400"
									)}>
										<SelectValue placeholder={placeholder} />
									</SelectTrigger>
									<SelectContent className="p-0">
										{filterable && (
											<div className="p-2 border-b">
												<Input
													placeholder={t("Placeholder.Search")}
													value={searchQuery}
													onChange={(e) => setSearchQuery(e.target.value)}
													className="focus-visible:ring-0 border-custom-primary-accent"
												/>
											</div>
										)}
										{/* Filtered Options */}
										<div className="max-h-[300px] overflow-y-auto">
											{options
												?.filter((_option: any) => {
													const displayName =
														locale === "ar" ? _option.ar : _option.en;
													return displayName
														.toLowerCase()
														.includes(searchQuery.toLowerCase());
												})
												.map((_option: any) => (
													<SelectItem key={_option.en} value={_option.en}>
														{locale === "ar" ? _option.ar : _option.en}
													</SelectItem>
												))}
											{options?.length === 0 && (
												<div className="py-2 text-center text-sm text-muted-foreground">
													{t("Placeholder.NoResultsFound")}
												</div>
											)}
										</div>
									</SelectContent>
								</Select>
							) : type === "date" ? (
								<Popover>
									<PopoverTrigger asChild>
										<Button
											variant="outline"
											className={cn(
												"h-[56px] w-full rounded-none border-[#DEDEDE] justify-start text-left font-normal",
												!field.value && "text-muted-foreground"
											)}
										>
											<CalendarIcon className={cn("h-4 w-4",
												locale === "ar" ? "ml-4" : "mr-4"
											)} color="black" />
											{field.value ? (
												format(new Date(field.value as string), "PPP")
											) : (
												<span>{placeholder}</span>
											)}
										</Button>
									</PopoverTrigger>
									<PopoverContent className="w-auto p-0" align="start">
										<CalendarWithSearch
											mode="single"
											selected={field.value ? new Date(field.value as string) : undefined}
											onSelect={(date) => field.onChange(date?.toISOString().split("T")[0])}
											initialFocus
											disabled={(date) => {
												if (name === "startDate") {
													return new Date(date as any) < new Date(new Date().setHours(0, 0, 0, 0));
												}
												else if (name === "dateOfBirth") {
													return new Date(date as any) > new Date(new Date().setHours(0, 0, 0, 0));
												}
												return false;
											}}
										/>
									</PopoverContent>
								</Popover>
							) : type === "tel" ? (
								<div className="col-span-12 rounded-md border border-gray-300 bg-white h-11 mt-3 p-2">
									<PhoneInput
										labelclassName="text-right"
										className="h-12 -mt-2"
										{...field}
										value={field.value?.toString() || ""}
										placeholder={placeholder || ""}
										onChange={field.onChange}
									/>
								</div>
							) : (
								<PlaceholdersAndVanishInput
									{...field}
									placeholders={[placeholder || ""]}
									onChange={field.onChange}
									className="h-[56px] w-full rounded-none border-[#DEDEDE] bg-white"
									locale={locale}
									icon={icon}
								/>
							)}
						</FormControl>
						<FormMessage />
					</FormItem>
				)}
			/>
		);
	};

	const onSubmit = async (values: EmploymentFormValues) => {

		try {
			setIsLoading(true);
			const formData = new FormData();
			formData.append("full_name", values.fullName);
			formData.append("email", values.email);
			formData.append("phone", values.phone);
			if (files.expertCertificates) formData.append("expert_certifications", files.expertCertificates);
			if (files.professionalLicenses) formData.append("professional_licenses", files.professionalLicenses);
			if (files.qualifications) formData.append("qualifications", files.qualifications);
			formData.append("experience_years", values.yearsOfExperience.toString());
			if (files.cv) formData.append("cv", files.cv);
			formData.append("address", values.address);
			formData.append("start_date", values.startDate);
			formData.append("expected_salary", values.expectedSalary.toString());
			formData.append("gender", values.gender);
			formData.append("nationality", values.nationality);
			formData.append("birth_date", values.dateOfBirth);

			const { success, message } = await submitRequestForEmploymentForm(formData);
			console.log("Response: ", success, message);
			if (success) {
				toast.success(t("Job application submitted successfully"), {
					style: {
						backgroundColor: "green",
						color: "white",
					},
				});

				setFiles({
					cv: null,
					expertCertificates: null,
					professionalLicenses: null,
					qualifications: null,
				});
				form.reset({
					fullName: "",
					email: "",
					phone: "",
					expertCertificates: null,
					professionalLicenses: null,
					qualifications: null,
					yearsOfExperience: "",
					cv: null,
					address: "",
					startDate: "",
					expectedSalary: "",
					gender: "male",
					nationality: "",
					dateOfBirth: "",
				});
			} else {
				toast.error(t("Something went wrong"));
			}
		} catch (error) {
			console.error("Error: ", error);
			toast.error(t("Something went wrong"));
		} finally {
			setIsLoading(false);
		}
	};

	return (
		<div>
			<Form {...form}>
				<form onSubmit={form.handleSubmit(onSubmit)} className="space-y-6">
					{renderFormField(
						"fullName",
						t("Label.FullName"),
						t("Placeholder.FullName"),
						"text",
						undefined,
						undefined,
						undefined,
						"lucide:user"
					)}
					{renderFormField(
						"email",
						t("Label.Email"),
						t("Placeholder.Email"),
						"email",
						undefined,
						undefined,
						undefined,
						"system-uicons:mail"
					)}
					{renderFormField(
						"phone",
						t("Label.Phone"),
						t("Placeholder.Phone"),
						"tel",
						undefined,
						undefined,
						undefined,
						"lineicons:phone"
					)}
					<FileField
						name="expertCertificates"
						label={t("Label.ExpertCertificates")}
					/>
					<FileField
						name="professionalLicenses"
						label={t("Label.ProfessionalLicenses")}
					/>
					<FileField name="qualifications" label={t("Label.Qualifications")} />
					<FileField name="cv" label={t("Label.CV")} />
					{renderFormField(
						"yearsOfExperience",
						t("Label.YearsOfExperience"),
						t("Placeholder.YearsOfExperience"),
						"number",
						undefined,
						undefined,
						undefined,
						"fluent-mdl2:calendar-year"
					)}
					{renderFormField(
						"address",
						t("Label.Address"),
						t("Placeholder.Address"),
						"text",
						undefined,
						undefined,
						undefined,
						"proicons:location"
					)}
					{renderFormField(
						"startDate",
						t("Label.StartDate"),
						t("Placeholder.StartDate"),
						"date",
						undefined,
						undefined,
						undefined,
						"lucide:calendar-days"
					)}
					{renderFormField(
						"expectedSalary",
						t("Label.ExpectedSalary"),
						t("Placeholder.ExpectedSalary"),
						"number",
						undefined,
						undefined,
						undefined,
						"fluent:money-16-regular"
					)}
					{renderFormField(
						"gender",
						t("Label.Gender"),
						undefined,
						"select",
						genders,
						undefined,
						undefined,
						"lucide:gender"
					)}
					{renderFormField(
						"nationality",
						t("Label.Nationality"),
						t("Placeholder.Nationality"),
						"select",
						nationalities,
						undefined,
						undefined,
						"lucide:flag",
						true
					)}
					{renderFormField(
						"dateOfBirth",
						t("Label.DateOfBirth"),
						t("Placeholder.DateOfBirth"),
						"date",
						undefined,
						undefined,
						undefined,
						"lucide:calendar-days"
					)}
					<button
						type="submit"
						className="h-12 w-full rounded bg-custom-primary-accent text-2xl font-normal text-white"
						disabled={isLoading}
					>
						{isLoading ?
							<div className="flex items-center justify-center">
								<Loader2 className="animate-spin" />
							</div>
							: (
								t("Button.Submit")
							)}
					</button>
				</form>
			</Form>
		</div>
	);
}
